/*
 * ExtCDMediaBSDClient class defintion
 *
 * Copyright (C) 2008-2010 Charles Davis
 *
 * This file is dual-licensed under the LGPL and BSD 3-clause license.
 * It is part of the XNU Kernel Extras project, which aims to extend the
 * XNU kernel beyond what is provided by Apple.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * ============================================================================
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. Neither the name of the author nor the names of any
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ExtCDMediaBSDClient.h"
#include "diskio_private.h"

int exec_scsi_cmd32( IOMediaBSDClient *client, dk_scsi_command_32_t *data, proc_t proc );
int exec_scsi_cmd64( IOMediaBSDClient *client, dk_scsi_command_64_t *data, proc_t proc );

/* ExtCDMediaBSDClient class implementation */
OSDefineMetaClassAndStructors( net_sf_driver_ExtCDMediaBSDClient, IOCDMediaBSDClient );

#define super IOCDMediaBSDClient
int net_sf_driver_ExtCDMediaBSDClient::ioctl( dev_t dev, u_long cmd, caddr_t data,
                                              int flags, proc_t task )
{
    switch (cmd)
    {
    case DKIOCPREVENT:
        if (getProvider()->getProperty( kIOMediaRemovableKey ) == kOSBooleanTrue)
            return getProvider()->errnoFromReturn( getProvider()->getProvider()->lockMedia( true ) );
        else return 0;
    case DKIOCALLOW:
        if (getProvider()->getProperty( kIOMediaRemovableKey ) == kOSBooleanTrue)
            return getProvider()->errnoFromReturn( getProvider()->getProvider()->lockMedia( false ) );
        else return 0;
    case DKIOCSCSICOMMAND32:
        return exec_scsi_cmd32( this, (dk_scsi_command_32_t *)data, task );
    case DKIOCSCSICOMMAND64:
        return exec_scsi_cmd64( this, (dk_scsi_command_64_t *)data, task );
    default:
        return super::ioctl( dev, cmd, data, flags, task );
    }
}

OSMetaClassDefineReservedUnused( net_sf_driver_ExtCDMediaBSDClient, 0 );
