#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <libkern/OSByteOrder.h>
#include <IOKit/scsi/SCSITask.h>
#include <IOKit/scsi/SCSICmds_INQUIRY_Definitions.h>

#include "wine/diskio.h"

int main(int argc, char** argv)
{
	dk_scsi_command_t cmd;
	SCSICmd_INQUIRY_StandardData inq_data;
	int fd = open(argv[1], O_RDWR);
	if(fd < 0)
	{
		fprintf(stderr, "%s: Couldn't open %s (%s)\n", argv[0], argv[1], strerror(errno));
		return 1;
	}

	memset(&cmd, 0, sizeof(cmd));
	cmd.attribute = kSCSITask_SIMPLE;
	cmd.timeout = 0;
	cmd.direction = kSCSIDataTransfer_FromTargetToInitiator;
	cmd.cdbSize = 6;
	cmd.cdb[0] = 0x12;	// INQUIRY
	cmd.cdb[1] = 0x00;
	cmd.cdb[2] = 0x00;
	OSWriteBigInt16(cmd.cdb, 3, (int16_t)(uint16_t)sizeof(inq_data));
	cmd.cdb[5] = 0x00;
	cmd.buffer = &inq_data;
	cmd.bufferSize = sizeof(inq_data);
	if(ioctl(fd, DKIOCSCSICOMMAND, &cmd) < 0)
	{
		fprintf(stderr, "%s: SCSI ioctl failed (%s)\n", argv[0], strerror(errno));
		close(fd);
		return 1;
	}
	close(fd);
	// Now to print the contents of the INQUIRY.
	printf("%s:\n", argv[1]);
	printf("\tPeripheral type: %2.2x\n", inq_data.PERIPHERAL_DEVICE_TYPE);
	printf("\tRemovable: %s\n", inq_data.RMB ? "true" : "false");
	printf("\tVersion: ISO: %1.1d, ECMA: %1.1d, ANSI: %1.1d\n", inq_data.VERSION >> 6, (inq_data.VERSION >> 3) & 0x7, inq_data.VERSION & 0x7);
	printf("\tVendor: %*s\n", kINQUIRY_VENDOR_IDENTIFICATION_Length, inq_data.VENDOR_IDENTIFICATION);
	printf("\tProduct: %*s\n", kINQUIRY_PRODUCT_IDENTIFICATION_Length, inq_data.PRODUCT_IDENTIFICATION);
	printf("\tRevision: %*s\n", kINQUIRY_PRODUCT_REVISION_LEVEL_Length, inq_data.PRODUCT_REVISION_LEVEL);
	return 0;
}

