/*
 * Hack to build SCSITask commands
 *
 * Copyright (C) 2010 Charles Davis
 *
 * This file is dual-licensed under the LGPL and BSD 3-clause license.
 * It is part of the XNU Kernel Extras project, which aims to extend the
 * XNU kernel beyond what is provided by Apple.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * ============================================================================
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. Neither the name of the author nor the names of any
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _EXTSCSITASKBUILDER_H
#define _EXTSCSITASKBUILDER_H

#if defined(KERNEL) && defined(__cplusplus)
# include <IOKit/scsi/IOSCSIPrimaryCommandsDevice.h>

/* ExtSCSITaskBuilder class definition */
/*
 * This is a gross hack used to get around the fact that the APIs for
 * creating SCSITask objects are protected members of the logical unit
 * driver family (which starts with IOSCSIPrimaryCommandsDevice).
 * I'm pretty sure Apple isn't going to like what I'm about to do...
 * but I really don't care because the SCSITaskUserClient API just
 * isn't adequate.
 * So, the SOLE PURPOSE of this class is to be a factory for SCSITask
 * objects, exposing the SCSITask API to clients like
 * ExtMediaBSDClient and its cousins.
 */
class net_sf_driver_ExtSCSITaskBuilder: private IOSCSIPrimaryCommandsDevice
{
    OSDeclareDefaultStructors( net_sf_driver_ExtSCSITaskBuilder );

protected:

    // Stub methods, mostly power-management related, that we don't care about.
    virtual UInt32 GetInitialPowerState( void );
    virtual void HandlePowerChange( void );
    virtual void HandleCheckPowerState( void );
    virtual void TicklePowerManager( void );

    virtual bool InitializeDeviceSupport( void );
    virtual void StartDeviceSupport( void );
    virtual void SuspendDeviceSupport( void );
    virtual void ResumeDeviceSupport( void );
    virtual void StopDeviceSupport( void );
    virtual void TerminateDeviceSupport( void );
    virtual UInt32 GetNumberOfPowerStateTransitions( void );
    virtual bool ClearNotReadyStatus( void );

public:

    using IOSCSIPrimaryCommandsDevice::GetSCSITask;
    using IOSCSIPrimaryCommandsDevice::ReleaseSCSITask;
    using IOSCSIPrimaryCommandsDevice::SetTaskAttribute;
    using IOSCSIPrimaryCommandsDevice::GetTaskStatus;
    using IOSCSIPrimaryCommandsDevice::SetCommandDescriptorBlock;
    using IOSCSIPrimaryCommandsDevice::SetDataTransferDirection;
    using IOSCSIPrimaryCommandsDevice::SetRequestedDataTransferCount;
    using IOSCSIPrimaryCommandsDevice::GetRealizedDataTransferCount;
    using IOSCSIPrimaryCommandsDevice::SetDataBuffer;
    using IOSCSIPrimaryCommandsDevice::SetTimeoutDuration;
    using IOSCSIPrimaryCommandsDevice::SetTaskCompletionCallback;
    using IOSCSIPrimaryCommandsDevice::GetServiceResponse;
    using IOSCSIPrimaryCommandsDevice::GetAutoSenseData;
};

#endif

#endif /* _EXTSCSITASKBUILDER_H */